#ifndef bqt_elemlist_hh
#define bqt_elemlist_hh

#include <set>
#include <list>

#include "elemlistbase.hh"

template<unsigned Size, typename T>
class elemlist : public elemlistbase
{
    class Elem
    {
        typedef std::set<T> listtype;
        listtype elems[Size];
        
    public:
        typename listtype::const_iterator begin(unsigned ind) const { return elems[ind].begin(); }
        typename listtype::const_iterator end(unsigned ind) const { return elems[ind].end(); }
        unsigned size(unsigned ind) const { return elems[ind].size(); }
#ifdef ELEMLIST_DEBUG
        void dump() const;
#endif
        bool iscommon(const Elem &b, unsigned ind) const;
        
        void copy(const Elem &b, unsigned ind);

        void combine(const Elem &b);
        void combine(const Elem &b, unsigned ind);

        bool contains(const Elem &b, bool mayopt=false) const;
        bool contains(const Elem &b, unsigned ind) const;
        const Elem difference(const Elem &b) const;
        void insert(unsigned ind, T elem);
        void addplot(T coord, ...);
    };
    
    typedef std::list<Elem> listtype;
    listtype elems;

    bool achieves(const elemlist &b, bool cruel=false) const;
    bool achieves(const elemlist &b, typename listtype::const_iterator without, bool cruel=false) const;

    elemlist(Elem &b, unsigned count=1) : elems(count, b) { }

public:
    void insert(const Elem &b) { elems.push_back(b); }
    unsigned size() const { return elems.size(); }
    
    elemlist() { }
    
#ifdef ELEMLIST_DEBUG
    virtual void dump() const;
#endif
    
    virtual void combine();              /* Groups algebraically. */
    virtual void expand();               /* Breaks the groups. */
    virtual void join();                 /* Decreases the amount of elements by overlapping. */
    virtual void inflate();              /* Inflates the elements. */
    virtual void crop(bool cruel=true);  /* Drop redundant elements. */
    
    template<class L3MAP>
    void expand3(const L3MAP &max3test);

    template<class fmtfunc>
    void format(fmtfunc &formatter) const;
};

#include "elemlist.tcc"

#endif
